# (c) Copyright 2016 Hewlett Packard Enterprise Development LP
#
#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.

from testtools import testcase

from barbican.tests import utils
from functionaltests.api import base
from functionaltests.api.v1.behaviors import secret_behaviors
from functionaltests.api.v1.behaviors import secretstores_behaviors
from functionaltests.common import config

CONF = config.get_config()
admin_a = CONF.rbac_users.admin_a
creator_a = CONF.rbac_users.creator_a
observer_a = CONF.rbac_users.observer_a
auditor_a = CONF.rbac_users.auditor_a
admin_b = CONF.rbac_users.admin_b
observer_b = CONF.rbac_users.observer_b

test_user_data_when_enabled = {
    'with_admin_a': {'user': admin_a, 'expected_return': 200},
    'with_creator_a': {'user': creator_a, 'expected_return': 403},
    'with_observer_a': {'user': observer_a, 'expected_return': 403},
    'with_auditor_a': {'user': auditor_a, 'expected_return': 403},
}

test_user_data_when_not_enabled = {
    'with_admin_a': {'user': admin_a, 'expected_return': 404},
    'with_creator_a': {'user': creator_a, 'expected_return': 403},
    'with_observer_a': {'user': observer_a, 'expected_return': 403},
    'with_auditor_a': {'user': auditor_a, 'expected_return': 403},
}


@utils.parameterized_test_case
class SecretStoresTestCase(base.TestCase):
    """Functional tests exercising ACL Features"""
    def setUp(self):
        super(SecretStoresTestCase, self).setUp()
        self.secret_behaviors = secret_behaviors.SecretBehaviors(self.client)
        self.ss_behaviors = secretstores_behaviors.SecretStoresBehaviors(
            self.client)

    def tearDown(self):
        self.ss_behaviors.cleanup_preferred_secret_store_entities()
        self.secret_behaviors.delete_all_created_secrets()
        super(SecretStoresTestCase, self).tearDown()

    def _validate_secret_store_fields(self, secret_store):
        self.assertIsNotNone(secret_store['name'])
        self.assertIsNotNone(secret_store['secret_store_ref'])
        self.assertIsNotNone(secret_store['secret_store_plugin'])
        self.assertIsNotNone(secret_store['global_default'])
        self.assertIsNotNone(secret_store['created'])
        self.assertIsNotNone(secret_store['updated'])
        self.assertEqual("ACTIVE", secret_store['status'])

    @testcase.skipUnless(base.conf_multiple_backends_enabled, 'executed only '
                         'when multiple backends support is enabled in '
                         'barbican server side')
    @utils.parameterized_dataset(test_user_data_when_enabled)
    def test_get_all_secret_stores_multiple_enabled(self, user,
                                                    expected_return):

        resp, json_data = self.ss_behaviors.get_all_secret_stores(
            user_name=user)

        self.assertEqual(expected_return, resp.status_code)
        if expected_return == 200:
            self.assertIsNotNone(json_data.get('secret_stores'))
            stores = json_data['secret_stores']
            for secret_store in stores:
                self._validate_secret_store_fields(secret_store)

    @testcase.skipIf(base.conf_multiple_backends_enabled, 'executed only when '
                     'multiple backends support is NOT enabled in barbican '
                     'server side')
    @utils.parameterized_dataset(test_user_data_when_not_enabled)
    def test_get_all_secret_stores_multiple_disabled(self, user,
                                                     expected_return):

        resp, _ = self.ss_behaviors.get_all_secret_stores(user_name=user)
        self.assertEqual(expected_return, resp.status_code)

    @testcase.skipUnless(base.conf_multiple_backends_enabled, 'executed only '
                         'when multiple backends support is enabled in '
                         'barbican server side')
    @utils.parameterized_dataset(test_user_data_when_enabled)
    def test_get_global_default_multiple_enabled(self, user, expected_return):

        resp, json_data = self.ss_behaviors.get_global_default(user_name=user)

        self.assertEqual(expected_return, resp.status_code)
        if expected_return == 200:
            self._validate_secret_store_fields(json_data)

    @testcase.skipIf(base.conf_multiple_backends_enabled, 'executed only when '
                     'multiple backends support is NOT enabled in barbican '
                     'server side')
    @utils.parameterized_dataset(test_user_data_when_not_enabled)
    def test_get_global_default_multiple_disabled(self, user, expected_return):

        resp, _ = self.ss_behaviors.get_global_default(user_name=user)
        self.assertEqual(expected_return, resp.status_code)

    @testcase.skipUnless(base.conf_multiple_backends_enabled, 'executed only '
                         'when multiple backends support is enabled in '
                         'barbican server side')
    @utils.parameterized_dataset(test_user_data_when_enabled)
    def test_get_project_preferred_multiple_enabled(self, user,
                                                    expected_return):

        resp, json_data = self.ss_behaviors.get_all_secret_stores(
            user_name=admin_a)
        self.assertEqual(200, resp.status_code)

        stores = json_data['secret_stores']

        store = stores[len(stores) - 1]
        secret_store_ref = store['secret_store_ref']
        resp = self.ss_behaviors.set_preferred_secret_store(secret_store_ref,
                                                            user_name=user)

        if resp.status_code == 204:
            resp, json_data = self.ss_behaviors.get_project_preferred_store(
                user_name=user)

            self.assertEqual(expected_return, resp.status_code)
            if expected_return == 200:
                self._validate_secret_store_fields(json_data)
                self.assertEqual(store['secret_store_ref'],
                                 json_data['secret_store_ref'])

    @testcase.skipIf(base.conf_multiple_backends_enabled, 'executed only when '
                     'multiple backends support is NOT enabled in barbican '
                     'server side')
    @utils.parameterized_dataset(test_user_data_when_not_enabled)
    def test_get_project_preferred_multiple_disabled(self, user,
                                                     expected_return):

        resp, _ = self.ss_behaviors.get_project_preferred_store(user_name=user)
        self.assertEqual(expected_return, resp.status_code)

    @testcase.skipUnless(base.conf_multiple_backends_enabled, 'executed only '
                         'when multiple backends support is enabled in '
                         'barbican server side')
    @utils.parameterized_dataset(test_user_data_when_enabled)
    def test_get_a_secret_store_multiple_enabled(self, user, expected_return):
        # read global default secret store via admin user as this is not a
        # global default API check.
        resp, json_data = self.ss_behaviors.get_global_default(
            user_name=admin_a)
        self.assertEqual(200, resp.status_code)

        resp, json_data = self.ss_behaviors.get_a_secret_store(
            json_data['secret_store_ref'], user_name=user)

        self.assertEqual(expected_return, resp.status_code)
        if expected_return == 200:
            self._validate_secret_store_fields(json_data)
            self.assertTrue(json_data['global_default'])

    @testcase.skipIf(base.conf_multiple_backends_enabled, 'executed only when '
                     'multiple backends support is NOT enabled in barbican '
                     'server side')
    @utils.parameterized_dataset(test_user_data_when_not_enabled)
    def test_get_a_secret_store_multiple_disabled(self, user, expected_return):

        resp, _ = self.ss_behaviors.get_project_preferred_store(user_name=user)
        self.assertEqual(expected_return, resp.status_code)

    @testcase.skipUnless(base.conf_multiple_backends_enabled, 'executed only '
                         'when multiple backends support is enabled in '
                         'barbican server side')
    @utils.parameterized_dataset(test_user_data_when_enabled)
    def test_unset_project_preferred_store_multiple_enabled(self, user,
                                                            expected_return):

        resp, json_data = self.ss_behaviors.get_all_secret_stores(
            user_name=admin_a)
        self.assertEqual(200, resp.status_code)

        stores = json_data['secret_stores']

        store = stores[len(stores) - 1]
        secret_store_ref = store['secret_store_ref']
        resp = self.ss_behaviors.set_preferred_secret_store(secret_store_ref,
                                                            user_name=user)

        if resp.status_code == 204:
            # after setting project preference, get preferred will return 200
            resp, json_data = self.ss_behaviors.get_project_preferred_store(
                user_name=user)
            self.assertEqual(200, resp.status_code)

            # now, remove project preferred secret store
            self.ss_behaviors.unset_preferred_secret_store(
                json_data['secret_store_ref'], user_name=user)
            # get project preferred call should now return 404
            resp, json_data = self.ss_behaviors.get_project_preferred_store(
                user_name=user)
            self.assertEqual(404, resp.status_code)



# Copyright (c) 2015 Cisco Systems
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from testtools import testcase

from barbican.tests import utils
from functionaltests.api import base
from functionaltests.api.v1.behaviors import acl_behaviors
from functionaltests.api.v1.behaviors import consumer_behaviors
from functionaltests.api.v1.behaviors import container_behaviors
from functionaltests.api.v1.behaviors import secret_behaviors
from functionaltests.api.v1.models import acl_models
from functionaltests.api.v1.models import consumer_model
from functionaltests.api.v1.models import container_models
from functionaltests.api.v1.models import secret_models
from functionaltests.common import config


CONF = config.get_config()
admin_a = CONF.rbac_users.admin_a
creator_a = CONF.rbac_users.creator_a
observer_a = CONF.rbac_users.observer_a
auditor_a = CONF.rbac_users.auditor_a
admin_b = CONF.rbac_users.admin_b
observer_b = CONF.rbac_users.observer_b


def get_rbac_only():
    return {'read': {'project-access': True}}


# private secret can only be access by the creator or an admin
def get_private():
    return {'read': {'project-access': False}}


def get_acl_only(reader_id):
    return {'read': {'users': [reader_id], 'project-access': False}}


def get_rbac_plus_acl(reader_id):
    return {'read': {'users': [reader_id], 'project-access': True}}


test_data_read_secret_rbac_only = {
    'with_admin_a': {'user': admin_a, 'expected_return': 200},
    'with_creator_a': {'user': creator_a, 'expected_return': 200},
    'with_observer_a': {'user': observer_a, 'expected_return': 200},
    'with_auditor_a': {'user': auditor_a, 'expected_return': 403},
    'with_admin_b': {'user': admin_b, 'expected_return': 403},
    'with_observer_b': {'user': observer_b, 'expected_return': 403},
}

test_data_read_secret_private = {
    'with_admin_a': {'user': admin_a, 'expected_return': 200},
    'with_creator_a': {'user': creator_a, 'expected_return': 200},
    'with_observer_a': {'user': observer_a, 'expected_return': 403},
    'with_auditor_a': {'user': auditor_a, 'expected_return': 403},
    'with_admin_b': {'user': admin_b, 'expected_return': 403},
    'with_observer_b': {'user': observer_b, 'expected_return': 403},
}

test_data_read_secret_acl_only = {
    'with_admin_a': {'user': admin_a, 'expected_return': 200},
    'with_creator_a': {'user': creator_a, 'expected_return': 200},
    'with_observer_a': {'user': observer_a, 'expected_return': 403},
    'with_auditor_a': {'user': auditor_a, 'expected_return': 403},
    'with_admin_b': {'user': admin_b, 'expected_return': 403},
    'with_observer_b': {'user': observer_b, 'expected_return': 200},
}

test_data_read_secret_rbac_plus_acl = {
    'with_admin_a': {'user': admin_a, 'expected_return': 200},
    'with_creator_a': {'user': creator_a, 'expected_return': 200},
    'with_observer_a': {'user': observer_a, 'expected_return': 200},
    'with_auditor_a': {'user': auditor_a, 'expected_return': 403},
    'with_admin_b': {'user': admin_b, 'expected_return': 403},
    'with_observer_b': {'user': observer_b, 'expected_return': 200},
}

test_data_read_container_rbac_only = {
    'with_admin_a': {'user': admin_a, 'expected_return': 200},
    'with_creator_a': {'user': creator_a, 'expected_return': 200},
    'with_observer_a': {'user': observer_a, 'expected_return': 200},
    'with_auditor_a': {'user': auditor_a, 'expected_return': 200},
    'with_admin_b': {'user': admin_b, 'expected_return': 403},
    'with_observer_b': {'user': observer_b, 'expected_return': 403},
}

test_data_read_container_private = {
    'with_admin_a': {'user': admin_a, 'expected_return': 200},
    'with_creator_a': {'user': creator_a, 'expected_return': 200},
    'with_observer_a': {'user': observer_a, 'expected_return': 403},
    'with_auditor_a': {'user': auditor_a, 'expected_return': 403},
    'with_admin_b': {'user': admin_b, 'expected_return': 403},
    'with_observer_b': {'user': observer_b, 'expected_return': 403},
}

test_data_read_container_acl_only = {
    'with_admin_a': {'user': admin_a, 'expected_return': 200},
    'with_creator_a': {'user': creator_a, 'expected_return': 200},
    'with_observer_a': {'user': observer_a, 'expected_return': 403},
    'with_auditor_a': {'user': auditor_a, 'expected_return': 403},
    'with_admin_b': {'user': admin_b, 'expected_return': 403},
    'with_observer_b': {'user': observer_b, 'expected_return': 200},
}

test_data_read_container_rbac_plus_acl = {
    'with_admin_a': {'user': admin_a, 'expected_return': 200},
    'with_creator_a': {'user': creator_a, 'expected_return': 200},
    'with_observer_a': {'user': observer_a, 'expected_return': 200},
    'with_auditor_a': {'user': auditor_a, 'expected_return': 200},
    'with_admin_b': {'user': admin_b, 'expected_return': 403},
    'with_observer_b': {'user': observer_b, 'expected_return': 200},
}

test_data_read_container_consumer_acl_only = {
    'with_admin_a': {'user': admin_a, 'expected_return': 200},
    'with_creator_a': {'user': creator_a, 'expected_return': 200},
    'with_observer_a': {'user': observer_a, 'expected_return': 200},
    'with_auditor_a': {'user': auditor_a, 'expected_return': 200},
    'with_admin_b': {'user': admin_b, 'expected_return': 200},
    'with_observer_b': {'user': observer_b, 'expected_return': 200},
}

test_data_delete_container_consumer_acl_only = {
    'with_admin_a': {'user': admin_a, 'expected_return': 200},
    'with_creator_a': {'user': creator_a, 'expected_return': 403},
    'with_observer_a': {'user': observer_a, 'expected_return': 403},
    'with_auditor_a': {'user': auditor_a, 'expected_return': 403},
    'with_admin_b': {'user': admin_b, 'expected_return': 403},
    'with_observer_b': {'user': observer_b, 'expected_return': 403},
}

test_data_create_container_consumer_acl_only = {
    'with_admin_a': {'user': admin_a, 'expected_return': 200},
    'with_creator_a': {'user': creator_a, 'expected_return': 403},
    'with_observer_a': {'user': observer_a, 'expected_return': 403},
    'with_auditor_a': {'user': auditor_a, 'expected_return': 403},
    'with_admin_b': {'user': admin_b, 'expected_return': 200},
    'with_observer_b': {'user': observer_b, 'expected_return': 403},
}


@utils.parameterized_test_case
class AclTestCase(base.TestCase):
    """Functional tests exercising ACL Features"""
    def setUp(self):
        super(AclTestCase, self).setUp()
        self.secret_behaviors = secret_behaviors.SecretBehaviors(self.client)
        self.container_behaviors = container_behaviors.ContainerBehaviors(
            self.client)
        self.acl_behaviors = acl_behaviors.AclBehaviors(self.client)
        self.consumer_behaviors = consumer_behaviors.ConsumerBehaviors(
            self.client)

    def tearDown(self):
        self.acl_behaviors.delete_all_created_acls()
        self.secret_behaviors.delete_all_created_secrets()
        self.container_behaviors.delete_all_created_containers()
        self.consumer_behaviors.delete_all_created_consumers()
        super(AclTestCase, self).tearDown()

    @utils.parameterized_dataset(test_data_read_secret_rbac_only)
    def test_secret_read_default(self, user, expected_return):
        secret_ref = self.store_secret()
        status = self.get_secret(secret_ref, user_name=user)
        self.assertEqual(expected_return, status)

    @utils.parameterized_dataset(test_data_read_secret_rbac_only)
    def test_secret_read_rbac_only(self, user, expected_return):
        secret_ref = self.store_secret()
        self.set_secret_acl(secret_ref, get_rbac_only())
        status = self.get_secret(secret_ref, user_name=user)
        self.assertEqual(expected_return, status)

    @utils.parameterized_dataset(test_data_read_secret_private)
    def test_secret_read_private(self, user, expected_return):
        secret_ref = self.store_secret()
        self.set_secret_acl(secret_ref, get_private())
        status = self.get_secret(secret_ref, user_name=user)
        self.assertEqual(expected_return, status)

    @utils.parameterized_dataset(test_data_read_secret_acl_only)
    def test_secret_read_acl_only(self, user, expected_return):
        secret_ref = self.store_secret()
        user_id = self.secret_behaviors.get_user_id_from_name(observer_b)
        self.set_secret_acl(secret_ref, get_acl_only(user_id))
        status = self.get_secret(secret_ref, user_name=user)
        self.assertEqual(expected_return, status)

    @utils.parameterized_dataset(test_data_read_secret_rbac_plus_acl)
    def test_secret_read_rbac_plus_acl(self, user, expected_return):
        secret_ref = self.store_secret()
        user_id = self.secret_behaviors.get_user_id_from_name(observer_b)
        self.set_secret_acl(secret_ref, get_rbac_plus_acl(user_id))
        status = self.get_secret(secret_ref, user_name=user)
        self.assertEqual(expected_return, status)

    @utils.parameterized_dataset(test_data_read_container_rbac_only)
    def test_container_read_default(self, user, expected_return):
        container_ref = self.store_container()
        status = self.get_container(container_ref, user_name=user)
        self.assertEqual(expected_return, status)

    @utils.parameterized_dataset(test_data_read_container_rbac_only)
    def test_container_read_rbac_only(self, user, expected_return):
        container_ref = self.store_container()
        self.set_container_acl(container_ref, get_rbac_only())
        status = self.get_container(container_ref, user_name=user)
        self.assertEqual(expected_return, status)

    @utils.parameterized_dataset(test_data_read_container_private)
    def test_container_read_private(self, user, expected_return):
        container_ref = self.store_container()
        self.set_container_acl(container_ref, get_private())
        status = self.get_container(container_ref, user_name=user)
        self.assertEqual(expected_return, status)

    @utils.parameterized_dataset(test_data_read_container_acl_only)
    def test_container_read_acl_only(self, user, expected_return):
        container_ref = self.store_container()
        user_id = self.container_behaviors.get_user_id_from_name(observer_b)
        self.set_container_acl(container_ref, get_acl_only(user_id))
        status = self.get_container(container_ref, user_name=user)
        self.assertEqual(expected_return, status)

    @utils.parameterized_dataset(test_data_read_container_rbac_plus_acl)
    def test_container_read_rbac_plus_acl(self, user, expected_return):
        container_ref = self.store_container()
        user_id = self.container_behaviors.get_user_id_from_name(observer_b)
        self.set_container_acl(container_ref, get_rbac_plus_acl(user_id))
        status = self.get_container(container_ref, user_name=user)
        self.assertEqual(expected_return, status)

    @utils.parameterized_dataset(test_data_read_container_consumer_acl_only)
    def test_container_acl_read_consumers(self, user, expected_return):
        """Acl access will not allow you to see the list of consumers"""
        container_ref = self.store_container(user_name=creator_a,
                                             admin=admin_a)
        consumer_model = get_consumer_model()

        resp, consumer_data = self.consumer_behaviors.create_consumer(
            model=consumer_model,
            container_ref=container_ref,
            user_name=admin_a)
        self.assertEqual(200, resp.status_code)

        user_id = self.container_behaviors.get_user_id_from_name(user)
        self.set_container_acl(container_ref, get_acl_only(user_id))

        # Verify all users granted acl access can read the container
        status_code = self.get_container(container_ref, user_name=user)
        self.assertEqual(200, status_code)

        resp, consumers, next_ref, prev_ref = \
            self.consumer_behaviors.get_consumers(container_ref,
                                                  user_name=user)

        self.assertEqual(expected_return, resp.status_code)

    @utils.parameterized_dataset(test_data_delete_container_consumer_acl_only)
    def test_container_acl_remove_consumer(self, user, expected_return):
        """Acl access will not allow you to delete a consumer"""
        container_ref = self.store_container(user_name=creator_a,
                                             admin=admin_a)
        consumer_model = get_consumer_model()

        resp, consumer_data = self.consumer_behaviors.create_consumer(
            model=consumer_model,
            container_ref=container_ref,
            user_name=admin_a)
        self.assertEqual(200, resp.status_code)

        user_id = self.container_behaviors.get_user_id_from_name(user)
        self.set_container_acl(container_ref, get_acl_only(user_id))

        # Verify all users granted acl access can read the container
        status_code = self.get_container(container_ref, user_name=user)
        self.assertEqual(200, status_code)

        resp, consumer_data = self.consumer_behaviors.delete_consumer(
            model=consumer_model,
            container_ref=container_ref,
            user_name=user)
        self.assertEqual(expected_return, resp.status_code)

    @utils.parameterized_dataset(test_data_create_container_consumer_acl_only)
    def test_container_acl_create_consumer(self, user, expected_return):
        """Acl access will not allow you to add a consumer"""
        container_ref = self.store_container(user_name=creator_a,
                                             admin=admin_a)

        user_id = self.container_behaviors.get_user_id_from_name(user)
        self.set_container_acl(container_ref, get_acl_only(user_id))

        # Verify all users granted acl access can read the container
        status_code = self.get_container(container_ref, user_name=user)
        self.assertEqual(200, status_code)

        consumer_model = get_consumer_model()

        resp, consumer_data = self.consumer_behaviors.create_consumer(
            model=consumer_model,
            container_ref=container_ref,
            user_name=user)
        self.assertEqual(expected_return, resp.status_code)

    @testcase.attr('negative')
    def test_secret_acl_auditor_with_acl_cannot_read(self):
        """Auditor granted access to a secret cannot read that secret"""

        secret_ref = self.store_secret()
        self.set_secret_acl(secret_ref, get_rbac_plus_acl(auditor_a))

        status_code = self.get_secret(secret_ref=secret_ref,
                                      user_name=auditor_a)
        self.assertEqual(403, status_code)

    @testcase.attr('negative')
    def test_secret_acl_put_as_observer(self):
        """Observer can not put to a secret when granted access via acl"""

        secret_no_payload = {
            "name": "AES key",
            "expiration": "2030-02-28T19:14:44.180394",
            "algorithm": "aes",
            "bit_length": 256,
            "mode": "cbc",
        }
        secret_model = secret_models.SecretModel(**secret_no_payload)
        resp, secret_ref = self.secret_behaviors.create_secret(
            model=secret_model,
            user_name=creator_a)

        self.set_secret_acl(secret_ref, get_rbac_plus_acl(observer_a))

        # Update
        payload = "gF6+lLoF3ohA9aPRpt+6bQ=="
        payload_content_type = "application/octet-stream"
        payload_content_encoding = "base64"

        update_resp = self.secret_behaviors.update_secret_payload(
            secret_ref,
            user_name=observer_a,
            payload=payload,
            payload_content_type=payload_content_type,
            payload_content_encoding=payload_content_encoding)
        self.assertEqual(403, update_resp.status_code)

# ----------------------- Secret ACL Tests ---------------------------

    @testcase.attr('negative', 'security')
    def test_secret_read_acl_no_token(self):
        secret_ref = self.store_secret()
        acl_ref = '{0}/acl'.format(secret_ref)
        resp = self.acl_behaviors.get_acl(acl_ref, use_auth=False)
        self.assertEqual(401, resp.status_code)

    @testcase.attr('negative', 'security')
    def test_secret_set_acl_no_token(self):
        secret_ref = self.store_secret()
        resp = self.set_secret_acl(secret_ref, get_rbac_only(), use_auth=False)
        self.assertEqual(401, resp.status_code)

    @testcase.attr('negative', 'security')
    def test_secret_delete_acl_no_token(self):
        secret_ref = self.store_secret()
        acl_ref = '{0}/acl'.format(secret_ref)
        resp = self.acl_behaviors.delete_acl(
            acl_ref, expected_fail=True, use_auth=False
        )
        self.assertEqual(401, resp.status_code)

    @testcase.attr('negative', 'security')
    def test_secret_update_acl_no_token(self):
        secret_ref = self.store_secret()
        acl_ref = '{0}/acl'.format(secret_ref)
        resp = self.set_secret_acl(secret_ref, get_rbac_only())
        self.assertEqual(200, resp.status_code)
        resp = self.acl_behaviors.update_acl(acl_ref, {}, use_auth=False)
        self.assertEqual(401, resp.status_code)

# ----------------------- Container ACL Tests ---------------------------

    @testcase.attr('negative', 'security')
    def test_container_read_acl_no_token(self):
        container_ref = self.store_container()
        acl_ref = '{0}/acl'.format(container_ref)
        resp = self.acl_behaviors.get_acl(acl_ref, use_auth=False)
        self.assertEqual(401, resp.status_code)

    @testcase.attr('negative', 'security')
    def test_container_set_acl_no_token(self):
        container_ref = self.store_container()
        resp = self.set_container_acl(
            container_ref, get_rbac_only(), use_auth=False
        )
        self.assertEqual(401, resp.status_code)

    @testcase.attr('negative', 'security')
    def test_container_delete_acl_no_token(self):
        container_ref = self.store_container()
        acl_ref = '{0}/acl'.format(container_ref)
        resp = self.acl_behaviors.delete_acl(
            acl_ref, expected_fail=True, use_auth=False
        )
        self.assertEqual(401, resp.status_code)

    @testcase.attr('negative', 'security')
    def test_container_update_acl_no_token(self):
        container_ref = self.store_container()
        acl_ref = '{0}/acl'.format(container_ref)
        resp = self.set_container_acl(container_ref, get_rbac_only())
        self.assertEqual(200, resp.status_code)
        resp = self.acl_behaviors.update_acl(acl_ref, {}, use_auth=False)
        self.assertEqual(401, resp.status_code)

# ----------------------- Helper Functions ---------------------------

    def store_secret(self, user_name=creator_a, admin=admin_a):
        test_model = secret_models.SecretModel(
            **get_default_secret_data())
        resp, secret_ref = self.secret_behaviors.create_secret(
            test_model, user_name=user_name, admin=admin)
        self.assertEqual(201, resp.status_code)
        return secret_ref

    def get_secret(self, secret_ref, user_name=creator_a):
        resp = self.secret_behaviors.get_secret(
            secret_ref, 'application/octet-stream',
            user_name=user_name)
        return resp.status_code

    def set_secret_acl(self, secret_ref, acl, use_auth=True,
                       user_name=creator_a):
        test_model = acl_models.AclModel(**acl)
        resp = self.acl_behaviors.create_acl(
            secret_ref, test_model, use_auth=use_auth, user_name=user_name)
        if use_auth:
            self.assertEqual(200, resp.status_code)
        return resp

    def store_container(self, user_name=creator_a, admin=admin_a):
        secret_ref = self.store_secret(user_name=user_name, admin=admin)

        test_model = container_models.ContainerModel(
            **get_container_req(secret_ref))
        resp, container_ref = self.container_behaviors.create_container(
            test_model, user_name=user_name, admin=admin)
        self.assertEqual(201, resp.status_code)
        return container_ref

    def get_container(self, container_ref, user_name=creator_a):
        resp = self.container_behaviors.get_container(
            container_ref, user_name=user_name)
        return resp.status_code

    def set_container_acl(self, container_ref, acl, use_auth=True,
                          user_name=creator_a):
        test_model = acl_models.AclModel(**acl)
        resp = self.acl_behaviors.create_acl(
            container_ref, test_model, use_auth=use_auth, user_name=user_name)
        if use_auth:
            self.assertEqual(200, resp.status_code)
        return resp

# ----------------------- Support Functions ---------------------------


def get_default_secret_data():
    return {
        "name": "AES key",
        "expiration": "2050-02-28T19:14:44.180394",
        "algorithm": "aes",
        "bit_length": 256,
        "mode": "cbc",
        "payload": get_default_payload(),
        "payload_content_type": "application/octet-stream",
        "payload_content_encoding": "base64",
    }


def get_default_payload():
    return 'Z0Y2K2xMb0Yzb2hBOWFQUnB0KzZiUT09'


def get_container_req(secret_ref):
    return {"name": "testcontainer",
            "type": "generic",
            "secret_refs": [{'name': 'secret1', 'secret_ref': secret_ref}]}


def get_consumer_model():
    test_consumer_model = consumer_model.ConsumerModel(
        name="consumername",
        URL="consumerURL"
    )

    return test_consumer_model



# Copyright 2011-2012 OpenStack LLC.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
import webob.exc

from barbican.api import middleware as mw
from barbican.common import config
from barbican.common import utils
import barbican.context
from barbican import i18n as u

LOG = utils.getLogger(__name__)
CONF = config.CONF


class BaseContextMiddleware(mw.Middleware):
    def process_request(self, req):
        request_id = req.headers.get('x-openstack-request-id')
        if not request_id:
            request_id = 'req-' + utils.generate_uuid()
        setattr(req, 'request_id', request_id)
        LOG.info('Begin processing request %(request_id)s',
                 {'request_id': request_id})

    def process_response(self, resp):

        resp.headers['x-openstack-request-id'] = resp.request.request_id

        LOG.info('Processed request: %(status)s - %(method)s %(url)s',
                 {"status": resp.status,
                  "method": resp.request.method,
                  "url": resp.request.url})
        return resp


class ContextMiddleware(BaseContextMiddleware):
    def __init__(self, app):
        super(ContextMiddleware, self).__init__(app)

    def process_request(self, req):
        """Convert authentication information into a request context

        Generate a barbican.context.RequestContext object from the available
        authentication headers and store on the 'context' attribute
        of the req object.

        :param req: wsgi request object that will be given the context object
        :raises webob.exc.HTTPUnauthorized: when value of the X-Identity-Status
                                            header is not 'Confirmed' and
                                            anonymous access is disallowed
        """
        super(ContextMiddleware, self).process_request(req)

        if req.headers.get('X-Identity-Status') == 'Confirmed':
            req.context = self._get_authenticated_context(req)
        elif CONF.allow_anonymous_access:
            req.context = self._get_anonymous_context()
            LOG.debug("==== Inserted barbican unauth "
                      "request context: %s ====", req.context.to_dict())
        else:
            raise webob.exc.HTTPUnauthorized()

        # Ensure that down wind mw.Middleware/app can see this context.
        req.environ['barbican.context'] = req.context

    def _get_anonymous_context(self):
        kwargs = {
            'user': None,
            'tenant': None,
            'is_admin': False,
            'read_only': True,
        }
        return barbican.context.RequestContext(**kwargs)

    def _get_authenticated_context(self, req):
        ctx = barbican.context.RequestContext.from_environ(req.environ)

        if ctx.project_id is None:
            LOG.debug("X_PROJECT_ID not found in request")
            return webob.exc.HTTPUnauthorized()

        ctx.is_admin = CONF.admin_role.strip().lower() in ctx.roles

        return ctx


class UnauthenticatedContextMiddleware(BaseContextMiddleware):
    def _get_project_id_from_header(self, req):
        project_id = req.headers.get('X-Project-Id')
        if not project_id:
            accept_header = req.headers.get('Accept')
            if not accept_header:
                req.headers['Accept'] = 'text/plain'
            raise webob.exc.HTTPBadRequest(detail=u._('Missing X-Project-Id'))

        return project_id

    def process_request(self, req):
        """Create a context without an authorized user."""
        super(UnauthenticatedContextMiddleware, self).process_request(req)

        project_id = self._get_project_id_from_header(req)

        config_admin_role = CONF.admin_role.strip().lower()
        roles_header = req.headers.get('X-Roles', '')
        roles = [r.strip().lower() for r in roles_header.split(',') if r]

        # If a role wasn't specified we default to admin
        if not roles:
            roles = [config_admin_role]

        kwargs = {
            'user': req.headers.get('X-User-Id'),
            'domain': req.headers.get('X-Domain-Id'),
            'user_domain': req.headers.get('X-User-Domain-Id'),
            'project_domain': req.headers.get('X-Project-Domain-Id'),
            'project_id': project_id,
            'roles': roles,
            'is_admin': config_admin_role in roles,
            'request_id': req.request_id
        }

        context = barbican.context.RequestContext(**kwargs)

        req.environ['barbican.context'] = context



#    Copyright 2018 Fujitsu.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from oslo_versionedobjects import base as object_base

from barbican.common import exception
from barbican import i18n as u
from barbican.model import models
from barbican.model import repositories as repos
from barbican.objects import base
from barbican.objects import fields


@object_base.VersionedObjectRegistry.register
class SecretStores(base.BarbicanObject, base.BarbicanPersistentObject,
                   object_base.VersionedObjectDictCompat):
    fields = {
        'store_plugin': fields.StringField(),
        'crypto_plugin': fields.StringField(nullable=True),
        'global_default': fields.BooleanField(default=False),
        'name': fields.StringField(),
        'status': fields.StringField(nullable=True, default=base.States.ACTIVE)
    }

    db_model = models.SecretStores
    db_repo = repos.get_secret_stores_repository()

    def _validate_fields(self, change_fields):
        msg = u._("Must supply non-Blank {0} argument for SecretStores entry.")

        if not change_fields.get('name'):
            raise exception.MissingArgumentError(msg.format("name"))
        if not change_fields.get('store_plugin'):
            raise exception.MissingArgumentError(msg.format("store_plugin"))

    @classmethod
    def get_all(cls, session=None):
        secret_stores_db = cls.db_repo.get_all(session)
        secret_stores_obj = [cls()._from_db_object(secret_store_db) for
                             secret_store_db in secret_stores_db]
        return secret_stores_obj



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import testtools

import barbican.plugin.interface.certificate_manager as cm
import barbican.plugin.simple_certificate_manager as simple


class WhenTestingSimpleCertificateManagerPlugin(testtools.TestCase):

    def setUp(self):
        super(WhenTestingSimpleCertificateManagerPlugin, self).setUp()
        self.plugin = simple.SimpleCertificatePlugin()

    def test_issue_certificate_request(self):
        result = self.plugin.issue_certificate_request(None, None, None, None)

        self.assertEqual(cm.CertificateStatus.WAITING_FOR_CA, result.status)

    def test_check_certificate_status(self):
        result = self.plugin.check_certificate_status(None, None, None, None)

        self.assertEqual(
            cm.CertificateStatus.CERTIFICATE_GENERATED, result.status)

    def test_modify_certificate_request(self):
        result = self.plugin.modify_certificate_request(None, None, None, None)

        self.assertEqual(cm.CertificateStatus.WAITING_FOR_CA, result.status)

    def test_cancel_certificate_request(self):
        result = self.plugin.cancel_certificate_request(None, None, None, None)

        self.assertEqual(cm.CertificateStatus.REQUEST_CANCELED, result.status)

    def test_supports(self):
        result = self.plugin.supports(None)

        self.assertTrue(result)

    def test_get_ca_info(self):
        result = self.plugin.get_ca_info()
        name = self.plugin.get_default_ca_name()
        self.assertIn(name, result)
        self.assertEqual(name, result[name][cm.INFO_NAME])
        self.assertEqual(self.plugin.get_default_signing_cert(),
                         result[name][cm.INFO_CA_SIGNING_CERT])

    def test_supported_request_types(self):
        result = self.plugin.supported_request_types()
        supported_list = [cm.CertificateRequestType.CUSTOM_REQUEST,
                          cm.CertificateRequestType.SIMPLE_CMC_REQUEST,
                          cm.CertificateRequestType.FULL_CMC_REQUEST,
                          cm.CertificateRequestType.STORED_KEY_REQUEST]
        self.assertEqual(supported_list, result)


class WhenTestingSimpleCertificateEventManagerPlugin(testtools.TestCase):

    def setUp(self):
        super(WhenTestingSimpleCertificateEventManagerPlugin, self).setUp()
        self.plugin = simple.SimpleCertificateEventPlugin()

    def test_notify_ca_is_unavailable(self):
        # Test that eventing plugin method does not have side effects such as
        #   raising exceptions.
        self.plugin.notify_ca_is_unavailable(None, None, None, None)

    def test_notify_certificate_is_ready(self):
        # Test that eventing plugin method does not have side effects such as
        #   raising exceptions.
        self.plugin.notify_certificate_is_ready(None, None, None)



# Copyright (c) 2015 Cisco Systems
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import six
import unittest

from barbican.common import exception as excep
from barbican.common import quota
from barbican.model import models
from barbican.tests import database_utils


class WhenTestingQuotaDriverFunctions(database_utils.RepositoryTestCase):

    def setUp(self):
        super(WhenTestingQuotaDriverFunctions, self).setUp()
        self.quota_driver = quota.QuotaDriver()

    def test_get_defaults(self):
        quotas = self.quota_driver._get_defaults()
        self.assertEqual(-1, quotas['secrets'])
        self.assertEqual(-1, quotas['orders'])
        self.assertEqual(-1, quotas['containers'])
        self.assertEqual(-1, quotas['consumers'])
        self.assertEqual(-1, quotas['cas'])

    def test_compute_effective_quotas_using_some_defaults(self):
        configured_quotas = {'consumers': None, 'containers': 66,
                             'orders': None, 'secrets': 55,
                             'cas': None}
        quotas = self.quota_driver._compute_effective_quotas(configured_quotas)
        expected_quotas = {'consumers': -1, 'containers': 66,
                           'orders': -1, 'secrets': 55,
                           'cas': -1}
        self.assertEqual(expected_quotas, quotas)

    def test_compute_effective_quotas_using_all_defaults(self):
        configured_quotas = {'consumers': None, 'containers': None,
                             'orders': None, 'secrets': None,
                             'cas': None}
        quotas = self.quota_driver._compute_effective_quotas(configured_quotas)
        expected_quotas = {'consumers': -1, 'containers': -1,
                           'orders': -1, 'secrets': -1,
                           'cas': -1}
        self.assertEqual(expected_quotas, quotas)

    def test_is_unlimited_true(self):
        self.assertTrue(self.quota_driver.is_unlimited_value(-1))

    def test_is_unlimited_false(self):
        self.assertFalse(self.quota_driver.is_unlimited_value(1))

    def test_is_disabled_true(self):
        self.assertTrue(self.quota_driver.is_disabled_value(0))

    def test_is_disabled_false(self):
        self.assertFalse(self.quota_driver.is_disabled_value(1))

    def test_should_get_project_quotas(self):
        self.create_a_test_project_quotas()
        project_quotas = self.quota_driver.get_project_quotas(
            self.get_test_project_id())
        self.assertEqual({'project_quotas':
                          self.get_test_parsed_project_quotas()},
                         project_quotas)

    def test_should_return_not_found_get_project_quotas(self):
        project_quotas = self.quota_driver.get_project_quotas('dummy')
        self.assertIsNone(project_quotas)

    def test_should_get_project_quotas_list(self):
        self.create_a_test_project_quotas()
        project_quotas = self.quota_driver.get_project_quotas_list()
        self.assertEqual({'project_quotas': [{
            'project_id': u'project1',
            'project_quotas': {'consumers': 105,
                               'containers': 103,
                               'orders': 102,
                               'secrets': 101,
                               'cas': 106}}], 'total': 1},
                         project_quotas)

    def test_should_get_empty_project_quotas_list(self):
        project_quotas = self.quota_driver.get_project_quotas_list()
        self.assertEqual({'total': 0, 'project_quotas': []}, project_quotas)

    def test_should_delete_project_quotas(self):
        self.create_a_test_project_quotas()
        self.quota_driver.delete_project_quotas(
            self.get_test_project_id())

    def test_should_raise_not_found_delete_project_quotas(self):
        self.assertRaises(
            excep.NotFound,
            self.quota_driver.delete_project_quotas,
            'dummy')

    def test_get_project_quotas_with_partial_definition(self):
        self.create_a_test_project_quotas('partial')
        project_quotas = self.quota_driver.get_project_quotas(
            self.get_test_project_id('partial'))
        self.assertEqual({'project_quotas':
                          self.get_test_response_project_quotas('partial')},
                         project_quotas)

    def test_get_project_quotas_using_empty_definition(self):
        self.create_a_test_project_quotas('none')
        project_quotas = self.quota_driver.get_project_quotas(
            self.get_test_project_id('none'))
        self.assertEqual({'project_quotas':
                          self.get_test_response_project_quotas('none')},
                         project_quotas)

    def test_get_quotas_using_some_defaults(self):
        self.create_a_test_project_quotas('partial')
        quotas = self.quota_driver.get_quotas(
            self.get_test_project_id('partial'))
        expected_quotas = {'quotas': {'consumers': -1, 'containers': 66,
                                      'orders': -1, 'secrets': 55,
                                      'cas': -1}}
        self.assertEqual(expected_quotas, quotas)

    def test_get_quotas_using_all_defaults(self):
        quotas = self.quota_driver.get_quotas('not_configured')
        expected_quotas = {'quotas': {'consumers': -1, 'containers': -1,
                                      'orders': -1, 'secrets': -1,
                                      'cas': -1}}
        self.assertEqual(expected_quotas, quotas)

    # ----------------------- Helper Functions ---------------------------
    def get_test_project_id(self, index=1):
        if index == 'partial':
            return 'project_partial'
        elif index == 'none':
            return 'project_none'
        else:
            return 'project' + str(index)

    def get_test_parsed_project_quotas(self, index=1):
        if index == 'partial':
            parsed_project_quotas = {
                'secrets': 55,
                'containers': 66}
        elif index == 'none':
            parsed_project_quotas = {}
        else:
            parsed_project_quotas = {
                'secrets': index * 100 + 1,
                'orders': index * 100 + 2,
                'containers': index * 100 + 3,
                'consumers': index * 100 + 5,
                'cas': index * 100 + 6}
        return parsed_project_quotas

    def get_test_response_project_quotas(self, index=1):
        if index == 'partial':
            response_project_quotas = {
                'secrets': 55,
                'orders': None,
                'containers': 66,
                'consumers': None,
                'cas': None}
        elif index == 'none':
            response_project_quotas = {
                'secrets': None,
                'orders': None,
                'containers': None,
                'consumers': None,
                'cas': None}
        else:
            response_project_quotas = {
                'secrets': index * 100 + 1,
                'orders': index * 100 + 2,
                'containers': index * 100 + 3,
                'consumers': index * 100 + 5,
                'cas': index * 100 + 6}
        return response_project_quotas

    def create_a_test_project_quotas(self, index=1):
        project_id = self.get_test_project_id(index)
        parsed_project_quotas = self.get_test_parsed_project_quotas(index)
        self.quota_driver.set_project_quotas(project_id, parsed_project_quotas)

    def create_project_quotas(self):
        for index in [1, 2, 3]:
            self.create_a_test_project_quotas(index)


class DummyRepoForTestingQuotaEnforcement(object):

    def __init__(self, get_count_return_value):
        self.get_count_return_value = get_count_return_value

    def get_count(self, internal_project_id):
        return self.get_count_return_value


class WhenTestingQuotaEnforcingFunctions(database_utils.RepositoryTestCase):

    def setUp(self):
        super(WhenTestingQuotaEnforcingFunctions, self).setUp()
        self.quota_driver = quota.QuotaDriver()
        self.project = models.Project()
        self.project.id = 'my_internal_id'
        self.project.external_id = 'my_keystone_id'

    def test_should_pass_default_unlimited(self):
        test_repo = DummyRepoForTestingQuotaEnforcement(0)
        quota_enforcer = quota.QuotaEnforcer('secrets', test_repo)
        quota_enforcer.enforce(self.project)

    def test_should_raise_disabled_value(self):
        test_repo = DummyRepoForTestingQuotaEnforcement(0)
        quota_enforcer = quota.QuotaEnforcer('secrets', test_repo)
        disabled_project_quotas = {'consumers': 0, 'containers': 0,
                                   'orders': 0, 'secrets': 0,
                                   'cas': 0}
        self.quota_driver.set_project_quotas(self.project.external_id,
                                             disabled_project_quotas)
        exception = self.assertRaises(
            excep.QuotaReached,
            quota_enforcer.enforce,
            self.project
        )
        self.assertIn('Quota reached for project', six.text_type(exception))
        self.assertIn('my_keystone_id', six.text_type(exception))
        self.assertIn('secrets', six.text_type(exception))
        self.assertIn(str(0), six.text_type(exception))

    def test_should_pass_below_limit(self):
        test_repo = DummyRepoForTestingQuotaEnforcement(4)
        quota_enforcer = quota.QuotaEnforcer('secrets', test_repo)
        five_project_quotas = {'consumers': 5, 'containers': 5,
                               'orders': 5, 'secrets': 5,
                               'cas': 5}
        self.quota_driver.set_project_quotas(self.project.external_id,
                                             five_project_quotas)
        quota_enforcer.enforce(self.project)

    def test_should_raise_equal_limit(self):
        test_repo = DummyRepoForTestingQuotaEnforcement(5)
        quota_enforcer = quota.QuotaEnforcer('secrets', test_repo)
        five_project_quotas = {'consumers': 5, 'containers': 5,
                               'orders': 5, 'secrets': 5,
                               'cas': 5}
        self.quota_driver.set_project_quotas(self.project.external_id,
                                             five_project_quotas)
        exception = self.assertRaises(
            excep.QuotaReached,
            quota_enforcer.enforce,
            self.project
        )
        self.assertIn('Quota reached for project', six.text_type(exception))
        self.assertIn('my_keystone_id', six.text_type(exception))
        self.assertIn('secrets', six.text_type(exception))
        self.assertIn(str(5), six.text_type(exception))

    def test_should_raise_above_limit(self):
        test_repo = DummyRepoForTestingQuotaEnforcement(6)
        quota_enforcer = quota.QuotaEnforcer('secrets', test_repo)
        five_project_quotas = {'consumers': 5, 'containers': 5,
                               'orders': 5, 'secrets': 5,
                               'cas': 5}
        self.quota_driver.set_project_quotas(self.project.external_id,
                                             five_project_quotas)
        exception = self.assertRaises(
            excep.QuotaReached,
            quota_enforcer.enforce,
            self.project
        )
        self.assertIn('Quota reached for project', six.text_type(exception))
        self.assertIn('my_keystone_id', six.text_type(exception))
        self.assertIn('secrets', six.text_type(exception))
        self.assertIn(str(5), six.text_type(exception))


if __name__ == '__main__':
    unittest.main()
